VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DataSet"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Public Enum DataSetEnum
    dsTrainingSet = 1
    dsTestingSet = 2
End Enum

Private mCurrentSet As DataSetEnum
Private trainingRows As Long
Private testingRows As Long
Private mColumns As Long

Private mErrNumber As Long
Private mError As String

Public Property Get ErrorNumber() As Long
    ErrorNumber = mErrNumber
End Property

Public Property Get ErrorDescription() As String
    
    If mErrNumber <> 0 Then
        ErrorDescription = mError
    End If
End Property

Public Property Get CurrentSet() As DataSetEnum
    CurrentSet = mCurrentSet
End Property

Public Property Get Columns() As Long
    Columns = mColumns
End Property
Public Property Get TrainingSamples() As Long
    TrainingSamples = trainingRows
End Property
Public Property Get TestingSamples() As Long
    TestingSamples = testingRows
End Property

Public Sub Initialise(ByRef RunPath As String, _
                      ByRef gxptTarget As Variant, _
                      ByRef gxptOutput As Variant, _
                      ByRef gxptParams As Variant, _
                      ByRef gxptModelInfo As Variant)
    
On Error GoTo errHandler

    mErrNumber = 0
        
    If Not DataSetIsLoaded Then
            
        If Not FileShared.FileExists(RunPath) Then
            Err.Raise 52000, "GXPT4CFHelper", "GeneXproTools run not found at: " & RunPath
        End If
        
        Dim conn As COMSqlite.Connection
        Dim rdr As COMSqlite.DataReader
        
        Set conn = New COMSqlite.Connection
        
        If conn.Open(RunPath) Then
            
            Set rdr = conn.ExecuteReader("SELECT * FROM RunParameters WHERE namespace = 'data' AND element = 'train_Samples'")
            If rdr.Read() Then
                trainingRows = rdr.GetInteger(5)
            End If
                        
            Set rdr = conn.ExecuteReader("SELECT * FROM RunParameters WHERE namespace = 'data' AND element = 'UseTestSet'")
            If rdr.Read() Then
                UseTest = rdr.GetInteger(5) = 1
            End If
            
            If UseTest Then
                Set rdr = conn.ExecuteReader("SELECT * FROM RunParameters WHERE namespace = 'data' AND element = 'test_Samples'")
                If rdr.Read() Then
                    testingRows = rdr.GetInteger(5)
                End If
            End If
            
                        
            Dim n As Long
            Dim Row As Long
            
            Set rdr = conn.ExecuteReader("SELECT * FROM TrainingSet ORDER BY row")
            mColumns = rdr.FieldCount - 1 'leave the "row" out
            
            ReDim TrainingSet(0 To Columns - 1, 0 To trainingRows - 1)
            Do While rdr.Read()
                For n = 0 To Columns - 1
                    TrainingSet(n, Row) = rdr.GetDouble(n)
                Next
                Row = Row + 1
            Loop
            
            If UseTest Then
                Set rdr = conn.ExecuteReader("SELECT * FROM TestingSet ORDER BY row")
                Row = 0
                
                ReDim TestingSet(0 To Columns - 1, 0 To testingRows - 1)
                Do While rdr.Read()
                    For n = 0 To Columns - 1
                        TestingSet(n, Row) = rdr.GetDouble(n)
                    Next
                    Row = Row + 1
                Loop
            End If
            
        End If
        
        Set rdr = Nothing
        If Not conn Is Nothing Then conn.Close
        Set conn = Nothing
        
        DataSetIsLoaded = True
    End If
    
    If gxptParams(0) = trainingRows Then
        mCurrentSet = dsTrainingSet
    Else
        mCurrentSet = dsTestingSet
    End If
    
errHandler:

    If Err Then
        If InStr(Err.Source, "COMSqlite") > 0 Then
            mError = "Error accessing the GeneXproTools run file."
        Else
            mError = Err.Description
        End If
        mErrNumber = Err.Number
    End If
End Sub

Public Function GetDoubleValue(ByVal Column As Long, ByVal Row As Long, ByVal DataSet As DataSetEnum)
    
    On Error GoTo errHandler
    
    If Not DataSetIsLoaded Then Exit Function
    
    If DataSet = dsTrainingSet Then
        GetDoubleValue = TrainingSet(Column, Row)
    Else
        GetDoubleValue = TestingSet(Column, Row)
    End If
    
errHandler:

    If Err Then
        mError = Err.Description
        mErrNumber = Err.Number
    End If
End Function
