using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace CSharpCustomFitness
{
    [ProgId("CSNETCF.MSE"), ClassInterface(ClassInterfaceType.AutoDispatch)]
    public class MSE
    {
        static GXPT4CFHelper.DataSet DataHelper = new GXPT4CFHelper.DataSet();

        public object Calculate(ref object gxptTarget, ref object gxptOutput, ref object gxptParams, ref object gxptModelInfo)
        {
            //This method must be called always. Internally it uses caching to minimize
            //any performance penalty. Replace the file below with a COPY of the run you are using.
            //This copy run must have the exact same Training and Testing sets and the same number
            //of samples as the run that is being evaluated.
            string runPath = @"c:\Documents and Settings\ze\My Documents\Acottest.gep";
            DataHelper.Initialise(ref runPath, ref gxptTarget, ref gxptOutput, ref gxptParams, ref gxptModelInfo);

            //These are examples of the interface available to you

            //This is how you access the values in the current set
            Debug.Print(DataHelper.GetDoubleValue(0, 0, GXPT4CFHelper.DataSetEnum.dsTrainingSet).ToString());

            //This property returns the set that is now being passed to this
            //fitness function. It relies on both sets having a different number of
            //samples so it does not work when both the Training and Testing set
            //have the same number of samples.
            Debug.Print(DataHelper.CurrentSet.ToString());

            //Returns the number of variables plus the dependent variable
            Debug.Print(DataHelper.Columns.ToString());

            //Returns the number of samples in use in the Training set
            Debug.Print(DataHelper.TrainingSamples.ToString());

            //Returns the number of samples in use in the Testing set
            Debug.Print(DataHelper.TestingSamples.ToString());

            //Returns the last error after a DataHelper call. You should
            //check if DataHelper.ErrorNumber is not zero to ensure that
            //it is returning a valida value.
            if(DataHelper.ErrorNumber > 0)
                Debug.Print(DataHelper.ErrorDescription);

            //-------------------------------------------------------------------
            //Implementation of the MSE fitness function
            object[] gxparams = (object[])gxptParams;
            object[] gxoutput = (object[])gxptOutput;
            object[] gxtarget = (object[])gxptTarget;

            int nSamples = (int)gxparams[0];
            double modelMinusTargetSquared = 0.0;
            double fitness = 0.0;
            double temp1 = 0.0;

            for(int i = 0; i < nSamples; i++)
            {
                temp1 = 0.0;
                temp1 = Convert.ToDouble(gxoutput[i]) - Convert.ToDouble(gxtarget[i]);
                temp1 *= temp1;
                modelMinusTargetSquared = modelMinusTargetSquared + temp1;
            }

            fitness = modelMinusTargetSquared / nSamples;

            if(fitness <= 0.000000001)
                fitness = 0.0;

            return (1 / (1 + fitness)) * 1000;
        }
    }
}
